<?php

$modules = array("sample", "sampledebtors", "nuvideo");

if (array_key_exists("do", $_POST) && $_POST["do"] == "1")
	do_setup();
else
{
	check_ini_settings();
	display_setup_form();
}


	/*** PRE-CONFIG CHECK FUNCTIONS ***/


function get_ini_boolean($setting_name)
{
	$value = ini_get($setting_name);
	return !(!$value || $value == "0" || strtolower($value) == "off");
} // func

function check_ini_settings()
{
	$ini_errors = "";
	if (get_ini_boolean("display_errors"))
		$ini_errors .= "<i>display_errors</i> needs to be set to <i>Off</i>.<br>";
	if (!get_ini_boolean("register_globals"))
		$ini_errors .= "<i>register_globals</i> needs to be set to <i>On</i>.<br>";
	if (get_ini_boolean("magic_quotes_gpc"))
		$ini_errors .= "<i>magic_quotes_gpc</i> needs to be set to <i>Off</i>.<br>";
	if (get_ini_boolean("magic_quotes_runtime"))
		$ini_errors .= "<i>magic_quotes_runtime</i> needs to be set to <i>Off</i>.<br>";
		
	$loaded_extensions = get_loaded_extensions();
	if (!in_array("mysql", $loaded_extensions))
		$ini_errors .= "The <i>mysql</i> extension needs to be loaded.<br>";
	if (!in_array("gd", $loaded_extensions) && !in_array("gd2", $loaded_extensions))
		$ini_errors .= "The <i>gd2</i> extension needs to be loaded.<br>";
	if (!in_array("curl", $loaded_extensions))
		$ini_errors .= "The <i>curl</i> extension needs to be loaded.<br>";

	if ($ini_errors)
	{
		$html = <<<EOHTML
			<table class="error">
				<tr>
					<td>
						In order for nuBuilder to work correctly, please configure your PHP installation as detailed below.<br>
						<br>
						{$ini_errors}
					</td>
				</tr>
			</table>
EOHTML;
		display($html);
		exit;
	} // if
} // func


	/*** END PRE-CONFIG CHECK FUNCTION ***/


	/*** PRE-SETUP CHECK FUNCTIONS ***/


function custom_escapeshellcmd($p_string)
{
	if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
		return str_replace("%", "^%", str_replace("^", "^^", $p_string));
	else
		return escapeshellcmd($p_string);
} // func


function fatal_input_error($p_message)
{
	display_setup_form(true, $p_message);
	die;
} // func


	
function check_input()
{
	$mysql_path = $_POST["mysql_path"];
	$php_path   = $_POST["php_path"];

	$process_output;
	exec("\"{$mysql_path}\" --version", $process_output);
	if (!$process_output)
		fatal_input_error("<i>{$mysql_path}</i> does not seem to be a valid MySQL!");
	
	$process_output = 0;
	exec("\"{$php_path}\" --version", $process_output);
	if (!$process_output)
		fatal_input_error("<i>{$php_path}</i> does not seem to be a valid PHP!");
	
	foreach ($GLOBALS["modules"] as $module_name)
	{
		$module_sql_path = $_POST["{$module_name}_sql_path"];
                $module_db       = $_POST["{$module_name}_db"];
                $module_user     = $_POST["{$module_name}_user"];
                $module_password = $_POST["{$module_name}_password"];

		$command_password_param = ($module_password) ? "-p".custom_escapeshellcmd($module_password) : "";
		$command = "".str_replace(" ", '" "', $mysql_path)." -u{$module_user} {$command_password_param} -e \"USE {$module_db}\"";

		$process_descriptors = array();
		$process_pipes;
		$process = proc_open($command, $process_descriptors, $process_pipes);
		if (!is_resource($process))
			fatal_input_error("Could not could not establish MySQL connection to {$module_db} database using user {$module_user}.");
		if (proc_close($process))
			fatal_input_error("Could not could not establish MySQL connection to {$module_db} database using user {$module_user}.");

		if (!is_file($module_sql_path))
			fatal_input_error("<i>{$module_sql_path}</i> does not exist or is not a file!");

		$config_filename = dirname(__FILE__)."/db/{$module_name}/config.php";
		$config_file_handle = @fopen($config_filename, "a");
		if (!$config_file_handle)
			fatal_input_error("Could not open <i>{$config_filename}</i> for writing!");
		fclose($config_file_handle);
	} // foreach

	$config_filename = dirname(__FILE__)."/productionnu2/config.php";
	$config_file_handle = @fopen($config_filename, "a");
	if (!$config_file_handle)
		fatal_input_error("Could not open <i>{$config_filename}</i> for writing!");
}


	/*** END PRE-SETUP CHECK FUNCTIONS ***/
	
	
	/*** SETUP FUNCTIONS ***/

	
function do_setup()
{
	check_input();

	$setup_filename = __FILE__;
	$setup_date     = date("Y/m/d H:i:s");

	$mysql_path = $_POST["mysql_path"];

	$warnings = "";

	foreach ($GLOBALS["modules"] as $module_name)
	{
		$module_sql_path = $_POST["{$module_name}_sql_path"];
		$module_db       = $_POST["{$module_name}_db"];
		$module_user     = $_POST["{$module_name}_user"];
		$module_password = $_POST["{$module_name}_password"];

		$_module_db                  = addslashes($module_db);
		$_module_user                = addslashes($module_user);
		$_module_password            = str_replace('$', '\$', addslashes($module_password));
		$_module_globeadmin_password = str_replace('$', '\$', addslashes($_POST["{$module_name}_globeadmin_password"]));

		$config_php_code = <<<EOTXT
<?php

/*
** This file was automatically generated by {$setup_filename} on {$setup_date}
*/

\$DBHost               = "127.0.0.1";
\$DBName               = "{$_module_db}";
\$DBUser               = "{$_module_user}";
\$DBPassword           = "{$_module_password}";
\$DBGlobeadminPassword = "{$_module_globeadmin_password}";

?>
EOTXT;
		$config_filename = dirname(__FILE__)."/db/{$module_name}/config.php";
		$config_file_handle = @fopen($config_filename, "w");
		if (!$config_file_handle)
		{
			$warnings .= "Could not open configuration file <i>{$config_filename}</i> for writing!<br>";
			$warnings .= "Skipping import of {$module_name} into {$module_db}...<br>";
			continue;
		} // if
		if (!fwrite($config_file_handle, $config_php_code))
		{
			$warnings .= "Could not write to configuration file <i>{$config_filename}</i>!<br>";
			$warnings .= "Skipping import of {$module_name} into {$module_db}...<br>";
			continue;
		} // if
		fclose($config_file_handle);

		$command_password_param = ($module_password) ? "-p".custom_escapeshellcmd($module_password) : "";
		$command = "\"{$mysql_path}\" -u{$module_user} {$command_password_param} {$module_db}";

		$process_pipe_descriptors = array(0 => array("file", $module_sql_path, "r"));
		$process_pipes;
		$process = proc_open($command, $process_pipe_descriptors, $process_pipes);
		if (!is_resource($process))
		{
			$warnings .= "Could not import database for <i>{$module_name}</i> using: <i>{$command}</i><br>";
			continue;
		} // if
		$process_output_code = proc_close($process);
		if ($process_output_code != 0)
		{
			$warnings .= "Could not import database for <i>{$module_name}</i> using: <i>{$command}</i><br>";
			continue;
		} // if
	} // foreach

	$_php_path            = addslashes($_POST["php_path"]);
	$_mysql_path          = addslashes($_POST["mysql_path"]);
	$_mail_method         = addslashes($_POST["mail_method"]);
	$_smtp_host           = addslashes($_POST["smtp_host"]);
	$_smtp_user           = addslashes($_POST["smtp_user"]);
	$_smtp_password       = str_replace('$', '\$', addslashes($_POST["smtp_password"]));
	$_globeadmin_password = str_replace('$', '\$', addslashes($_POST["globeadmin_password"]));

	$config_php_code = <<<EOTXT
<?php

/*
** This file was automatically generated by {$setup_filename} on {$setup_date}
*/

\$NUPHPExecutable      = "{$_php_path}";
\$NUMySQLExecutable    = "{$_mysql_path}";
\$NUMailMethod         = "{$_mail_method}";
\$NUSMTPHost           = "{$_smtp_host}";
\$NUSMTPUser           = "{$_smtp_user}";
\$NUSMTPPassword       = "{$_smtp_password}";
\$NUGlobeadminPassword = "{$_globeadmin_password}";

?>
EOTXT;
	$config_filename = dirname(__FILE__)."/productionnu2/config.php";
	$config_file_handle = @fopen($config_filename, "w");
	if (!$config_file_handle)
		$warnings .= "Could not open configuration file <i>{$config_filename}</i> for writing!<br>";
	else if (!fwrite($config_file_handle, $config_php_code))
		$warnings .= "Could not write to configuration file <i>{$config_filename}</i>!<br>";
	else
		fclose($config_file_handle);

	?><html>
<head>
	<title>nuBuilder Setup</title>
	<style type="text/css">
		* {
			font-family: sans-serif;
		}

		.warning {
			padding:            0.3em 0.6em;
			text-align:		    left;
			font-size:          75%;
			background-color:	#FFCC66;
			border:             1px solid black;
			width:				800px;
		}

		.success {
			padding:            0.3em 0.6em;
			text-align:		    left;
			font-size:          75%;
			background-color:	#00CC66;
			border:             1px solid black;
			width:				800px;
		}
	</style>
</head>
<body>
	<center><?php

	if ($warnings)
		echo		"<div class=\"warning\">{$warnings}</div><br>";

	echo <<<EOHTML
				<div class="success">Setup is now complete. Please delete or rename this file.</div>
			</center>
		</body>
		</html>
EOHTML;
} // func


	/*** END SETUP FUNCTIONS ***/
	
	
	/*** HTML GENERATION FUNCTIONS ***/
	
	
function display($p_content)
{
	?><html>
		<head>
			<title>nuBuilder Setup</title>
			<style type="text/css">
				* {
					font-family:        sans-serif;
				}
				
				td {
					padding:            0.3em 0.6em;
					font-size:          75%;
					font-weight:        700;
				}
				
				.error
				{
					text-align:		    left;
					font-weight:		400;
					background-color:	#FF0066;
					border:             1px solid black;
				}

				.content
				{
					border-collapse:    collapse;
					margin-top:         2em;
					background-color:   #2AE;
					border:             1px solid black;
				}

				.section_title
				{
					font-size:          large;
					font-weight:        bold;
				}

				.description
				{
					background-color:   #6CF;
					border-right:       none;
					border-bottom:      1px solid #19D;
					border-top:         1px solid #19D;					
					text-align:         right;
				}
				
				.value
				{
					border-left:        none;
					border-bottom:      1px solid #19D;
					border-top:         1px solid #19D;					
				}
			</style>
		</head>
		<body>
			<center>
				<?php echo $p_content ?>
			</center>
		</body>
		</html><?php
} // func



function display_setup_form($p_use_post_values = false, $p_error_message = "")
{
	@include_once("productionnu2/config.php");

	if ($p_use_post_values)
	{
		$detected_mysql_path         = $_POST["mysql_path"];
		$detected_php_path           = $_POST["php_path"];
		$mail_method                 = $_POST["mail_method"];
		$smtp_host                   = $_POST["smtp_host"];
		$smtp_user                   = $_POST["smtp_user"];
		$smtp_password               = $_POST["smtp_password"];
		$confirm_smtp_password       = $_POST["confirm_smtp_password"];
		$globeadmin_password         = $_POST["globeadmin_password"];
		$confirm_globeadmin_password = $_POST["confirm_globeadmin_password"];
	} // if
	else
	{
		$detected_mysql_path         = $NUMySQLExecutable ? $NUMySQLExecutable : shell_exec("which mysql");
		$detected_php_path           = $NUPHPExecutable   ? $NUPHPExecutable   : shell_exec("which php");
		$mail_method                 = $NUMailMethod;
		$smtp_host                   = $NUSMTPHost;
		$smtp_user                   = $NUSMTPUser;
		$smtp_password               = $NUSMTPPassword;
                $confirm_smtp_password       = $NUSMTPPassword;
		$globeadmin_password         = $NUGlobeadminPassword;
		$confirm_globeadmin_password = $NUGlobeadminPassword;
	} // else

	$html = <<<EOHTML
		<script type="text/javascript">
			function confirmPasswords()
			{
				if (document.getElementById("smtp_password").value != document.getElementById("confirm_smtp_password").value)
				{
					alert("The SMTP password does not match the confirm password.");
					return false;
				} // if
				if (document.getElementById("globeadmin_password").value != document.getElementById("confirm_globeadmin_password").value)
				{
					alert("The globeadmin password in the server settings section does not match the confirm password.");
					return false;
				} // if

EOHTML;

	foreach ($GLOBALS["modules"] as $module_name)
	{
		$html .= <<<EOHTML
				if (document.getElementById("{$module_name}_password").value != document.getElementById("{$module_name}_confirm_password").value)
				{
					alert("The database password for the site {$module_name} does not match the confirm password.");
					return false;
				} // if
				if (document.getElementById("{$module_name}_globeadmin_password").value != document.getElementById("{$module_name}_confirm_globeadmin_password").value)
				{
					alert("The globeadmin password for the site {$module_name} does not match the confirm password.");
					return false;
				} // if
				if (document.getElementById("globeadmin_password").value == "" && document.getElementById("{$module_name}_globeadmin_password").value == "")
				{
					alert("Neither the system-wide globeadmin password or the {$module_name} globeadmin password have been set. Please set one of these passwords to continue.");
					return false;
				} // if

EOHTML;
	} // foreach

	$html .= <<<EOHTML
				return true;
			} // func
		</script>
		<form method="post" action="setup.php">
			<input id="do" name="do" type="hidden" value="1">
			<table style="max-width: 800px">
EOHTML;

	if ($p_error_message)
	{
		$html .= <<<EOHTML
				<tr>
					<td>
						<table width="100%">
							<tr>
								<td class="error">{$p_error_message}</td>
							</tr>
						</table>
					</td>
				</tr>
				<tr>
					<td></td>
				</tr>
EOHTML;
	} // if

	$html .= <<<EOHTML
				<tr>
					<td>
						<table class="content">
							<tr>
								<td class="section_title" colspan="2">nuBuilder configuration</td>
							</tr>
							<tr>
								<td class="description">MySQL path</td>
								<td class="value"><input id="mysql_path" name="mysql_path" type="text" size="60" value="{$detected_mysql_path}"></td>
							</tr>
							<tr>
								<td class="description">PHP path</td>
								<td class="value"><input id="php_path" name="php_path" type="text" size="60" value="{$detected_php_path}"></td>
							</tr>
							<tr>
								<td class="description">Email Method</td>
								<td class="value">
									<select id="mail_method" name="mail_method">
EOHTML;

	foreach (array("mail", "smtp", "sendmail") as $value)
	{
		if ($value == $mail_method)
			$html .=		"<option selected=\"selected\">{$value}</option>";
		else
			$html .=		"<option>{$value}</option>";
	} // foreach

	$html .= <<<EOHTML
									</select>
								</td>
							</tr>
							<tr>
								<td class="description">SMTP host</td>
								<td class="value"><input id="smtp_host" name="smtp_host" type="text" value="{$smtp_host}"></td>
							</tr>
							<tr>
								<td class="description">SMTP user</td>
								<td class="value"><input id="smtp_user" name="smtp_user" type="text" value="{$smtp_user}"></td>
							</tr>
							<tr>
								<td class="description">SMTP password</td>
								<td class="value"><input id="smtp_password" name="smtp_password" type="password" value="{$smtp_password}"></td>
							</tr>
											<tr>
													<td class="description">Confirm SMTP password</td>
													<td class="value"><input id="confirm_smtp_password" name="confirm_smtp_password" type="password" value="{$confirm_smtp_password}"></td>
											</tr>
							<tr>
								<td class="description">System-wide globeadmin password</td>
								<td class="value"><input id="globeadmin_password" name="globeadmin_password" type="password" value="{$globeadmin_password}"></td>
							</tr>
							<tr>
								<td class="description">Confirm system-wide globeadmin password</td>
								<td class="value"><input id="confirm_globeadmin_password" name="confirm_globeadmin_password" type="password" value="{$confirm_globeadmin_password}"></td>
							</tr>
EOHTML;

	$detected_modules = array();
	foreach ($GLOBALS["modules"] as $module_name)
	{
		$filename = dirname(__FILE__)."/$module_name.sql";
		$detected_modules[$module_name] = (is_file($filename)) ? $filename : "";
	} // for

	foreach ($detected_modules as $module_name => $module_path)
	{
		@include_once("db/$module_name/config.php");

		if ($p_use_post_values)
		{
			$module_path                    = $_POST["{$module_name}_sql_path"];
			$db_name                        = $_POST["{$module_name}_db"];
			$db_user                        = $_POST["{$module_name}_user"];
			$db_password                    = $_POST["{$module_name}_password"];
            $db_confirm_password            = $_POST["{$module_name}_confirm_password"];
			$db_globeadmin_password         = $_POST["{$module_name}_globeadmin_password"];
			$db_confirm_globeadmin_password = $_POST["{$module_name}_confirm_globeadmin_password"];
		} // if
		else
		{
			$module_path                    = $module_path;
			$db_name                        = $DBName;
			$db_user                        = $DBUser;
			$db_password                    = $DBPassword;
            $db_confirm_password            = $DBPassword;
			$db_globeadmin_password         = $DBGlobeadminPassword;
			$db_confirm_globeadmin_password = $DBGlobeadminPassword;
		} // if

		$html .= <<<EOHTML
							<tr>
								<td class="section_title" colspan="2"><i>{$module_name}</i> site configuration</td>
							</tr>
							<tr>
								<td class="description">Path to SQL file</td>
								<td class="value"><input id="{$module_name}_sql_path" name="{$module_name}_sql_path" type="text" size="60" value="{$module_path}"></td>
							</tr>
							<tr>
								<td class="description">Database</td>
								<td class="value"><input id="{$module_name}_db" name="{$module_name}_db" type="text" value="{$db_name}"></td>
							</tr>
							<tr>
								<td class="description">Database user</td>
								<td class="value"><input id="{$module_name}_user" name="{$module_name}_user" type="text" value="{$db_user}"></td>
							</tr>
							<tr>
								<td class="description">Database user password</td>
								<td class="value"><input id="{$module_name}_password" name="{$module_name}_password" type="password" value="{$db_password}"></td>
							</tr>
											<tr>
													<td class="description">Confirm database user password</td>
													<td class="value"><input id="{$module_name}_confirm_password" name="{$module_name}_confirm_password" type="password" value="{$db_confirm_password}"></td>
											</tr>
							<tr>
								<td class="description">Site globeadmin password</td>
								<td class="value"><input id="{$module_name}_globeadmin_password" name="{$module_name}_globeadmin_password" type="password" value="{$db_globeadmin_password}"></td>
							</tr>
							<tr>
								<td class="description">Confirm site globeadmin password</td>
								<td class="value"><input id="{$module_name}_confirm_globeadmin_password" name="{$module_name}_confirm_globeadmin_password" type="password" value="{$db_confirm_globeadmin_password}"></td>
							</tr>
EOHTML;
	}

	$html .= <<<EOHTML
							<tr>
								<td colspan="2" align="center"><input type="button" value="Submit" onclick="if (confirmPasswords()) document.forms[0].submit();"></td>
							</tr>
						</table>
					</td>
				</tr>
			</table>
		</form>
EOHTML;
	display($html);
} // func


	/*** END HTML GENERATION FUNCTIONS ***/

?>